﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.ModelBinding;
using Kendo.Mvc.UI;
using Kreta.BusinessLogic.Classes;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Logic;
using Kreta.BusinessLogic.Security;
using Kreta.Client.KGR;
using Kreta.Core;
using Kreta.Enums;
using Kreta.Framework.Util;
using Kreta.Resources;
using Kreta.Web.Areas.Tanulo.Helper;
using Kreta.Web.Areas.Tanulo.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Helpers.Error;
using Kreta.Web.Helpers.Grid;
using Kreta.Web.Security;
using Newtonsoft.Json;

namespace Kreta.Web.Areas.Tanulo.ApiControllers
{
    [ApiRoleClaimsAuthorize(true)]
    [ApiRolePackageAuthorize(KretaClaimPackages.Adminisztrator.ClaimValue)]
    public class SzakkepzesiJuttatasokApiController : ApiController
    {
        private readonly IKGRClient kgrClient;

        public SzakkepzesiJuttatasokApiController(IKGRClient kgrClient)
        {
            this.kgrClient = kgrClient ?? throw new ArgumentNullException(nameof(kgrClient));
        }

        public DataSourceResult GetTanuloSzakkepzesiOsztondijGrid(string data, DataSourceRequest request)
        {
            var (gridParameter, modelList) = GetTanuloSzakkepzesiOsztondijGridData(data, request);

            modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

            return modelList.ToDataSourceResult(gridParameter);
        }

        private (GridParameters gridParameter, List<TanuloSzakkepzesiOsztondijGridModel> modelList) GetTanuloSzakkepzesiOsztondijGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloSzakkepzesiOsztondijSearchModel>(data);
            var gridParameter = Converter.GridParameter(request);

            var keresesCo = model.ConvertModelToCo();
            keresesCo.FeladatKategoriaId = OktNevelesiKategoriaEnum.NevelesOktatas.AsInt();
            keresesCo.LekerdezesNapja = DateTime.Today;

            var coList = new TanuloHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetSzakkepzesiOsztondijCoList(keresesCo);

            var modelList = new List<TanuloSzakkepzesiOsztondijGridModel>();
            foreach (var co in coList)
            {
                var gridModel = new TanuloSzakkepzesiOsztondijGridModel(co);
                modelList.Add(gridModel);
            }

            return (gridParameter, modelList);
        }

        public DataSourceResult GetTanuloEgyszeriPalyakezdesiGrid(string data, DataSourceRequest request)
        {
            var (gridParameter, modelList) = GetEgyszeriPalyakezdesiJuttatasGridData(data, request);

            modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

            return modelList.ToDataSourceResult(gridParameter);
        }

        public HttpResponseMessage GetSzakkepzesiOsztondijExport(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetTanuloSzakkepzesiOsztondijGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<TanuloSzakkepzesiOsztondijGridModel>(TanuloSzakkepzesiOsztondijGridModel.SzakkepzesiOsztondijExportAttributeId);

                var memoryStream = SzakkepzesiJuttatasokLogic.GetSzakkepzesiOszetondijExport(SzakkepzesiJuttatasokResource.ExportSzakkepzesiOsztondijSheetName, simpleExportColumnCos, modelList);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{SzakkepzesiJuttatasokResource.ExportTanuloSzakkepzesiOsztondijFilename}{DateTime.Now:yyyy_MM_dd_HHmm}{SzakkepzesiJuttatasokResource.ExportTanuloFileExtension}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        public HttpResponseMessage GetEgyszeriPalyakezdesiJuttatasExport(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetEgyszeriPalyakezdesiJuttatasGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<TanuloEgyszeriPalyakezdesiJuttatasGridModel>(TanuloSzakkepzesiOsztondijGridModel.EgyszeriPalyakezdesiJuttatasExportAttributeId);

                var memoryStream = SzakkepzesiJuttatasokLogic.GetEgyszeriPalyakezdesiJuttatasExport(SzakkepzesiJuttatasokResource.ExportEgyszeriPalyakezdesiJuttatasSheetName, simpleExportColumnCos, modelList);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{SzakkepzesiJuttatasokResource.ExportTanuloEgyszeriPalyakezdesiJutattasFilename}{DateTime.Now:yyyy_MM_dd_HHmm}{SzakkepzesiJuttatasokResource.ExportTanuloFileExtension}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        private (GridParameters gridParameter, List<TanuloEgyszeriPalyakezdesiJuttatasGridModel> modelList) GetEgyszeriPalyakezdesiJuttatasGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloEgyszeriPalyakezdesiJuttatasSearchModel>(data);
            var gridParameter = Converter.GridParameter(request);

            var keresesCo = model.ConvertModelToCo();
            keresesCo.FeladatKategoriaId = OktNevelesiKategoriaEnum.NevelesOktatas.AsInt();
            keresesCo.LekerdezesNapja = DateTime.Today;

            var coList = new TanuloHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetEgyszeriPalyakezdesiJuttatasCoList(keresesCo);

            var modelList = new List<TanuloEgyszeriPalyakezdesiJuttatasGridModel>();
            foreach (var co in coList)
            {
                var gridModel = new TanuloEgyszeriPalyakezdesiJuttatasGridModel(co);
                modelList.Add(gridModel);
            }

            return (gridParameter, modelList);
        }

        public DataSourceResult GetTanuloApaczaiIgenylesekGrid(string data, [ModelBinder(typeof(ModelBinder.DataSourceRequestModelBinder))] DataSourceRequest request)
        {
            var (gridParameter, modelList) = GetApaczaiIgenylesekGridData(data, request);

            modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

            return modelList.ToDataSourceResult(gridParameter);
        }

        public HttpResponseMessage GetApaczaiIgenylesekExport(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetApaczaiIgenylesekGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<TanuloApaczaiOsztondijIgenylesekGridModel>(TanuloApaczaiOsztondijBaseGridModel.ApaczaiIgenylesekExportAttributeId);

                var memoryStream = SimpleExportLogic.GetExport(SzakkepzesiJuttatasokResource.ExportApaczaiIgenylesekSheetName, simpleExportColumnCos, modelList, ClaimData.SelectedTanevID.Value);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{SzakkepzesiJuttatasokResource.ExportApaczaiIgenylesekFilename}{DateTime.Now:yyyy_MM_dd_HHmm}{SzakkepzesiJuttatasokResource.ExportTanuloFileExtension}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        private (GridParameters gridParameter, List<TanuloApaczaiOsztondijIgenylesekGridModel> modelList) GetApaczaiIgenylesekGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloApaczaiOsztondijIgenylesekSearchModel>(data);

            var gridParameter = Converter.GridParameter(request);

            var coList = new TanuloApaczaiHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetIgenylesekCoList(kgrClient, model.ConvertToCo(), ClaimData.IntezmenyGuid.Value);

            var modelList = new List<TanuloApaczaiOsztondijIgenylesekGridModel>();
            foreach (var co in coList)
            {
                var gridModel = new TanuloApaczaiOsztondijIgenylesekGridModel(co);
                modelList.Add(gridModel);
            }

            return (gridParameter, modelList);
        }

        public DataSourceResult GetTanuloApaczaiTanuloiListaGrid(string data, [ModelBinder(typeof(ModelBinder.DataSourceRequestModelBinder))] DataSourceRequest request)
        {
            var (gridParameter, modelList) = GetApaczaiTanuloiListaGridData(data, request);

            modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

            return modelList.ToDataSourceResult(gridParameter);
        }

        public HttpResponseMessage GetApaczaiTanuloiListaExport(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetApaczaiTanuloiListaGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<TanuloApaczaiOsztondijTanuloiListaGridModel>(TanuloApaczaiOsztondijBaseGridModel.ApaczaiTanuloiListaExportAttributeId);

                var memoryStream = SimpleExportLogic.GetExport(SzakkepzesiJuttatasokResource.ExportApaczaiTanuloiListaSheetName, simpleExportColumnCos, modelList, ClaimData.SelectedTanevID.Value);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{SzakkepzesiJuttatasokResource.ExportApaczaiTanuloiListaFilename}{DateTime.Now:yyyy_MM_dd_HHmm}{SzakkepzesiJuttatasokResource.ExportTanuloFileExtension}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        private (GridParameters gridParameter, List<TanuloApaczaiOsztondijTanuloiListaGridModel> modelList) GetApaczaiTanuloiListaGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloApaczaiOsztondijTanuloiListaSearchModel>(data);

            var keresesCo = model.ConvertModelToCo();
            keresesCo.FeladatKategoriaId = OktNevelesiKategoriaEnum.NevelesOktatas.AsInt();
            keresesCo.LekerdezesNapja = DateTime.Today;

            var gridParameter = Converter.GridParameter(request);

            var coList = new TanuloApaczaiHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetTanuloiListaCoList(kgrClient, keresesCo, ClaimData.IntezmenyGuid.Value);

            var modelList = new List<TanuloApaczaiOsztondijTanuloiListaGridModel>();
            modelList.AddRange(coList.Select(co => new TanuloApaczaiOsztondijTanuloiListaGridModel(co)));

            return (gridParameter, modelList);
        }

        [ApiRolePackageDenyAuthorize(KretaClaimPackages.Adminisztrator.ClaimValue)]
        public DataSourceResult GetTanuloApaczaiKifizetesekGrid(string data, DataSourceRequest request)
        {
            var (gridParameter, modelList) = GetApaczaiKifizetesekGridData(data, request);

            modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

            return modelList.ToDataSourceResult(gridParameter);
        }

        [ApiRolePackageDenyAuthorize(KretaClaimPackages.Adminisztrator.ClaimValue)]
        public HttpResponseMessage GetApaczaiKifizetesekExport(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetApaczaiKifizetesekGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<TanuloApaczaiOsztondijKifizetesekGridModel>(TanuloApaczaiOsztondijBaseGridModel.ApaczaiKifizetesekExportAttributeId);

                var memoryStream = SimpleExportLogic.GetExport(SzakkepzesiJuttatasokResource.ExportApaczaiKifizetésekSheetName, simpleExportColumnCos, modelList, ClaimData.SelectedTanevID.Value);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{SzakkepzesiJuttatasokResource.ExportApaczaiKifizetesekFilename}{DateTime.Now:yyyy_MM_dd_HHmm}{SzakkepzesiJuttatasokResource.ExportTanuloFileExtension}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        private (GridParameters gridParameter, List<TanuloApaczaiOsztondijKifizetesekGridModel> modelList) GetApaczaiKifizetesekGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloApaczaiOsztondijKifizetesekSearchModel>(data);

            var gridParameter = Converter.GridParameter(request);

            var coList = new TanuloApaczaiHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetKifizetesekCoList(kgrClient, model.ConvertToCo());

            var modelList = new List<TanuloApaczaiOsztondijKifizetesekGridModel>();
            foreach (var co in coList)
            {
                var gridModel = new TanuloApaczaiOsztondijKifizetesekGridModel(co);
                modelList.Add(gridModel);
            }

            return (gridParameter, modelList);
        }
    }
}
