﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.ModelBinding;
using Kendo.Mvc.UI;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Logic;
using Kreta.BusinessLogic.Security;
using Kreta.Core;
using Kreta.Framework.Util;
using Kreta.Resources;
using Kreta.Web.Areas.Tanulo.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Helpers.Error;
using Kreta.Web.Helpers.Grid;
using Kreta.Web.Security;
using Newtonsoft.Json;

namespace Kreta.Web.Areas.Tanulo.ApiControllers
{
    [ApiRoleClaimsAuthorize(true)]
    [ApiRolePackageAuthorize(KretaClaimPackages.Tanulo.ClaimValue, KretaClaimPackages.Gondviselo.ClaimValue)]
    public class TanuloHaziFeladatApiController : ApiController
    {
        public DataSourceResult GetTanulotHaziFeladatGrid(string data, [ModelBinder(typeof(ModelBinder.DataSourceRequestModelBinder))] DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloHaziFeladatSearchModel>(data);
            var co = model.ConvertModelToCo();

            var helper = new DktFeladatHelper(ConnectionTypeExtensions.GetSessionConnectionType());
            helper.GridParameters = Converter.GridParameter(request);
            var ds = helper.GetTanuloHaziFeladat(co, null, false, ClaimData.IsTanuloOrGondviselo);
            return ds.ToDataSourceResult();
        }

        public HttpResponseMessage GetHazifeladatExportGrid(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<TanuloHaziFeladatGridModel>(TanuloHaziFeladatGridModel.TanuloHaziFeladatExportAttributeId);

                var memoryStream = SimpleExportLogic.GetExport(ImportExportCommonResource.HaziFeladatok, simpleExportColumnCos, modelList, ClaimData.SelectedTanevID.Value);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{OrarendResource.HazifeladatExportFileName}_{DateTime.Now:yyyy_MM_dd}.{Constants.ImportExport.FileFormatXlsx}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        private (GridParameters gridParameter, List<TanuloHaziFeladatGridModel> modelList) GetGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloHaziFeladatSearchModel>(data);
            var gridParameter = Converter.GridParameter(request);
            var co = model.ConvertModelToCo();

            var helper = new DktFeladatHelper(ConnectionTypeExtensions.GetSessionConnectionType());
            var ds = helper.GetTanuloHaziFeladat(co, null, false, ClaimData.IsTanuloOrGondviselo);

            var modelList = new List<TanuloHaziFeladatGridModel>();
            foreach (var row in ds.Tables[0].AsEnumerable())
            {
                var gridModel = new TanuloHaziFeladatGridModel(row);
                modelList.Add(gridModel);
            }

            return (gridParameter, modelList);
        }
    }
}
