﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.ModelBinding;
using Kendo.Mvc.UI;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Logic;
using Kreta.BusinessLogic.Security;
using Kreta.Core;
using Kreta.Framework.Util;
using Kreta.Resources;
using Kreta.Web.Areas.Tanulo.Models;
using Kreta.Web.Helpers;
using Kreta.Web.Helpers.Error;
using Kreta.Web.Helpers.Grid;
using Kreta.Web.Security;
using Newtonsoft.Json;

namespace Kreta.Web.Areas.Tanulo.ApiControllers
{
    [ApiRoleClaimsAuthorize(true)]
    [ApiRolePackageAuthorize(KretaClaimPackages.Adminisztrator.ClaimValue)]
    public class TanuloJogviszonyAdatokApiController : ApiController
    {
        [HttpPost]
        public DataSourceResult GetJogviszonyAdatokGrid([ModelBinder(typeof(ModelBinder.DataSourceRequestModelBinder))] KretaGridDataSourceRequest request)
        {
            var (gridParameter, modelList) = GetGridData(request.data, request);

            return modelList.ToDataSourceResult(gridParameter);
        }

        public HttpResponseMessage GetExport(string data, DataSourceRequest request)
        {
            try
            {
                var (gridParameter, modelList) = GetGridData(data, request);

                modelList = modelList.SortingAndPaging(gridParameter.OrderDictionary);

                var simpleExportColumnCos = SimpleExportLogic.GetSimpleExportColumnCos<JogviszonyAdatokGridModel>(JogviszonyAdatokGridModel.TanulokJogviszonyaAdataiExportAttributeId);

                var memoryStream = SimpleExportLogic.GetExport(TanuloResource.TanuloJogviszonyAdataiExportSheetName, simpleExportColumnCos, modelList, ClaimData.SelectedTanevID.Value);

                return HttpResponseExtensions.GetFileHttpResponse(memoryStream.ToArray(), $"{TanuloResource.TanuloJogviszonyAdataiExportSheetName}_{DateTime.Now:yyyy_MM_dd}.{Constants.ImportExport.FileFormatXlsx}");
            }
            catch (Exception ex)
            {
                throw new StatusError(HttpStatusCode.BadRequest, ErrorResource.HibaTortentAFajlExportalasaKozben) { UnHandledException = ex };
            }
        }

        private (GridParameters gridParameter, List<JogviszonyAdatokGridModel> modelList) GetGridData(string data, DataSourceRequest request)
        {
            var model = JsonConvert.DeserializeObject<TanuloJogviszonyAdatokSearchModel>(data);

            var gridParameter = Converter.GridParameter(request);

            var coList = new TanuloHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetTanuloJogviszonyAdatokCoList(model.ConvertToCo());

            var modelList = new List<JogviszonyAdatokGridModel>();

            foreach (var co in coList)
            {
                var gridModel = new JogviszonyAdatokGridModel(co);
                modelList.Add(gridModel);
            }

            return (gridParameter, modelList);
        }
    }
}
